#!/usr/bin/env python
#

from gimpfu import *

############################################################################
# include, then use: erro_msg 
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    gimp.message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
############################################################################
#

def RotatingRGBColours (img, layer, sequence) :
    #Turn one colour in a layer to grey, without modifying the other two (RGB or RGBA).
    
    pdb.gimp_context_push()
    # Indicates that the process has started.
    gimp.progress_init("Reducing blueish tone in " + layer.name + "...")

    # Set up an undo group, so the operation will be undone in one step.
    pdb.gimp_image_undo_group_start(img)

    # Get the layer position.
    pos = 0;
    for i in range(len(img.layers)):
        if(img.layers[i] == layer):
            pos = i
    
    # Create a new layer to save the results (otherwise is not possible to undo the operation).
    newLayer = gimp.Layer(img, layer.name + " temp", layer.width, layer.height, layer.type, layer.opacity, layer.mode)
    img.add_layer(newLayer, pos)
    layerName = layer.name
    if '.' in layerName:
        split = layerName.split(".", 1)
        layerName = split[0]
    
    # Clear the new layer.
    pdb.gimp_edit_clear(newLayer)
    newLayer.flush()
    
    # Convert the pixels to gray scale.
    try:
        # Calculate the number of tiles.
        tn = int(layer.width / 64)
        if(layer.width % 64 > 0):
            tn += 1
        tm = int(layer.height / 64)
        if(layer.height % 64 > 0):
            tm += 1
        
        # Iterate over the tiles.
        for i in range(tn):
            for j in range(tm):
                # Update the progress bar.
                gimp.progress_update(float(i*tm + j) / float(tn*tm))
        
                # Get the tiles.
                srcTile = layer.get_tile(False, j, i)
                dstTile = newLayer.get_tile(False, j, i)
        
                # Iterate over the pixels of each tile.
                for x in range(srcTile.ewidth):
                    for y in range(srcTile.eheight):
                        # Get the pixel and calculate his gray value.
                        pixel = srcTile[x,y]
                        red = ord(pixel[0])
                        green = ord(pixel[1])
                        blue = ord(pixel[2])
                        gray = (ord(pixel[0]) + ord(pixel[1]) + ord(pixel[2]))/3
                        # chosen sequence
                        # Luminance (perceived option 1): (0.299*R + 0.587*G + 0.114*B)
                        if sequence == 0:       #GBR
                            newred = blue
                            newgreen = red
                            newblue = green
                        elif sequence == 1:     #BRG
                            newred = green
                            newgreen = blue
                            newblue = red
                        elif sequence == 2:     #RBG
                            newred = red
                            newgreen = blue
                            newblue = green
                        elif sequence == 3:     #GRB
                            newred = green
                            newgreen = red
                            newblue = blue
                        else:                   #BGR
                            newred = blue
                            newgreen = green
                            newblue = red
                        #res = chr(gray) + chr(gray) + chr(gray)
                        res = chr(newred) + chr(newgreen) + chr(newblue)
                        
                        # If the image has an alpha channel (or any other channel) copy his values.
                        if(len(pixel) > 3):
                            for k in range(len(pixel)-3):
                                res += pixel[k+3]
                                
                        # Save the value in the result layer.
                        dstTile[x,y] = res
        
        # Update the new layer.
        newLayer.flush()
        newLayer.merge_shadow (True)
        newLayer.update (0, 0, newLayer.width, newLayer.height)
        suffix = ["GBR","BRG","RBG","GRB","BGR"]
        
        # Remove the old layer (or keep it).
        # img.remove_layer(layer)
        
        # Change the name of the new layer (two layers can not have the same name).
        newLayer.name = layerName+" rotated "+suffix[sequence]
    except Exception as err:
        erro_msg ("Unexpected error: " + str(err))
    
    # Close the undo group.
    pdb.gimp_image_undo_group_end(img)
    
    # End progress.
    pdb.gimp_progress_end()
    pdb.gimp_context_pop()
    
    return(newLayer)

register(
    "RotatingRGBColours",
    "RotatingRGBColours",
    "Rotating RGB Colours.",
    "JFM-DN",
    "Open source (BSD 3-clause license)",
    "2017",
    "RotatingRGBColours",
    "RGB, RGB*",
    [
      (PF_IMAGE, "image", "Input image", None),
      (PF_DRAWABLE, "drawable", "Input drawable", None),
      (PF_OPTION, "sequence", "From RGB to:", 0, ("GBR", "BRG", "RBG", "GRB", "BGR")),
    ],
    [
      (PF_DRAWABLE, "outdrawable", "Output drawable", None),
    ],
    RotatingRGBColours,
    menu="<Image>/Diego/Ancillary"
    )

main()
